@@ -1,5 +1,3 @@
-# coding: utf-8
-
 import asyncio
 import hashlib
 import io
@@ -10,7 +8,6 @@ from http.cookies import BaseCookie, Morsel, SimpleCookie
 from unittest import mock
 
 import pytest
-from async_generator import async_generator, yield_
 from multidict import CIMultiDict, CIMultiDictProxy, istr
 from yarl import URL
 
@@ -22,6 +19,7 @@ from aiohttp.client_reqrep import (
     Fingerprint,
     _merge_ssl_params,
 )
+from aiohttp.helpers import PY_310
 from aiohttp.test_utils import make_mocked_coro
 
 
@@ -122,11 +120,6 @@ def test_version_err(make_request) -> None:
         make_request("get", "http://python.org/", version="1.c")
 
 
-def test_https_proxy(make_request) -> None:
-    with pytest.raises(ValueError):
-        make_request("get", "http://python.org/", proxy=URL("https://proxy.org"))
-
-
 def test_keep_alive(make_request) -> None:
     req = make_request("get", "http://python.org/", version=(0, 9))
     assert not req.keep_alive()
@@ -282,10 +275,17 @@ def test_host_header_ipv6_with_port(make_request) -> None:
     assert req.headers["HOST"] == "[::2]:99"
 
 
+@pytest.mark.xfail(
+    PY_310,
+    reason="No idea why ClientRequest() is constructed out of loop but "
+    "it calls `asyncio.get_event_loop()`",
+    raises=DeprecationWarning,
+)
 def test_default_loop(loop) -> None:
     asyncio.set_event_loop(loop)
     req = ClientRequest("get", URL("http://python.org/"))
     assert req.loop is loop
+    loop.run_until_complete(req.close())
 
 
 def test_default_headers_useragent(make_request) -> None:
@@ -306,7 +306,7 @@ def test_default_headers_useragent_custom(make_request) -> None:
 
 def test_skip_default_useragent_header(make_request) -> None:
     req = make_request(
-        "get", "http://python.org/", skip_auto_headers=set([istr("user-agent")])
+        "get", "http://python.org/", skip_auto_headers={istr("user-agent")}
     )
 
     assert "User-Agent" not in req.headers
@@ -591,6 +591,8 @@ async def test_connection_header(loop, conn) -> None:
     await req.send(conn)
     assert req.headers.get("CONNECTION") == "close"
 
+    await req.close()
+
 
 async def test_no_content_length(loop, conn) -> None:
     req = ClientRequest("get", URL("http://python.org"), loop=loop)
@@ -613,6 +615,7 @@ async def test_content_type_auto_header_get(loop, conn) -> None:
     resp = await req.send(conn)
     assert "CONTENT-TYPE" not in req.headers
     resp.close()
+    await req.close()
 
 
 async def test_content_type_auto_header_form(loop, conn) -> None:
@@ -622,6 +625,7 @@ async def test_content_type_auto_header_form(loop, conn) -> None:
     resp = await req.send(conn)
     assert "application/x-www-form-urlencoded" == req.headers.get("CONTENT-TYPE")
     resp.close()
+    await req.close()
 
 
 async def test_content_type_auto_header_bytes(loop, conn) -> None:
@@ -629,6 +633,7 @@ async def test_content_type_auto_header_bytes(loop, conn) -> None:
     resp = await req.send(conn)
     assert "application/octet-stream" == req.headers.get("CONTENT-TYPE")
     resp.close()
+    await req.close()
 
 
 async def test_content_type_skip_auto_header_bytes(loop, conn) -> None:
@@ -642,6 +647,7 @@ async def test_content_type_skip_auto_header_bytes(loop, conn) -> None:
     resp = await req.send(conn)
     assert "CONTENT-TYPE" not in req.headers
     resp.close()
+    await req.close()
 
 
 async def test_content_type_skip_auto_header_form(loop, conn) -> None:
@@ -655,19 +661,22 @@ async def test_content_type_skip_auto_header_form(loop, conn) -> None:
     resp = await req.send(conn)
     assert "CONTENT-TYPE" not in req.headers
     resp.close()
+    await req.close()
 
 
 async def test_content_type_auto_header_content_length_no_skip(loop, conn) -> None:
-    req = ClientRequest(
-        "post",
-        URL("http://python.org"),
-        data=io.BytesIO(b"hey"),
-        skip_auto_headers={"Content-Length"},
-        loop=loop,
-    )
-    resp = await req.send(conn)
-    assert req.headers.get("CONTENT-LENGTH") == "3"
-    resp.close()
+    with io.BytesIO(b"hey") as file_handle:
+        req = ClientRequest(
+            "post",
+            URL("http://python.org"),
+            data=file_handle,
+            skip_auto_headers={"Content-Length"},
+            loop=loop,
+        )
+        resp = await req.send(conn)
+        assert req.headers.get("CONTENT-LENGTH") == "3"
+        resp.close()
+        await req.close()
 
 
 async def test_urlencoded_formdata_charset(loop, conn) -> None:
@@ -681,6 +690,7 @@ async def test_urlencoded_formdata_charset(loop, conn) -> None:
     assert "application/x-www-form-urlencoded; charset=koi8-r" == req.headers.get(
         "CONTENT-TYPE"
     )
+    await req.close()
 
 
 async def test_post_data(loop, conn) -> None:
@@ -717,6 +727,7 @@ async def test_pass_falsy_data_file(loop, tmpdir) -> None:
     )
     assert req.headers.get("CONTENT-LENGTH", None) is not None
     await req.close()
+    testfile.close()
 
 
 # Elasticsearch API requires to send request body with GET-requests
@@ -915,6 +926,7 @@ async def test_expect100(loop, conn) -> None:
     assert req._continue is not None
     req.terminate()
     resp.close()
+    await req.close()
 
 
 async def test_expect_100_continue_header(loop, conn) -> None:
@@ -926,13 +938,13 @@ async def test_expect_100_continue_header(loop, conn) -> None:
     assert req._continue is not None
     req.terminate()
     resp.close()
+    await req.close()
 
 
 async def test_data_stream(loop, buf, conn) -> None:
-    @async_generator
     async def gen():
-        await yield_(b"binary data")
-        await yield_(b" result")
+        yield b"binary data"
+        yield b" result"
 
     req = ClientRequest("POST", URL("http://python.org/"), data=gen(), loop=loop)
     assert req.chunked
@@ -971,30 +983,31 @@ async def test_data_stream_deprecated(loop, buf, conn) -> None:
 
 
 async def test_data_file(loop, buf, conn) -> None:
-    req = ClientRequest(
-        "POST",
-        URL("http://python.org/"),
-        data=io.BufferedReader(io.BytesIO(b"*" * 2)),
-        loop=loop,
-    )
-    assert req.chunked
-    assert isinstance(req.body, payload.BufferedReaderPayload)
-    assert req.headers["TRANSFER-ENCODING"] == "chunked"
+    with io.BufferedReader(io.BytesIO(b"*" * 2)) as file_handle:
+        req = ClientRequest(
+            "POST",
+            URL("http://python.org/"),
+            data=file_handle,
+            loop=loop,
+        )
+        assert req.chunked
+        assert isinstance(req.body, payload.BufferedReaderPayload)
+        assert req.headers["TRANSFER-ENCODING"] == "chunked"
 
-    resp = await req.send(conn)
-    assert asyncio.isfuture(req._writer)
-    await resp.wait_for_close()
-    assert req._writer is None
-    assert buf.split(b"\r\n\r\n", 1)[1] == b"2\r\n" + b"*" * 2 + b"\r\n0\r\n\r\n"
-    await req.close()
+        resp = await req.send(conn)
+        assert asyncio.isfuture(req._writer)
+        await resp.wait_for_close()
+
+        assert req._writer is None
+        assert buf.split(b"\r\n\r\n", 1)[1] == b"2\r\n" + b"*" * 2 + b"\r\n0\r\n\r\n"
+        await req.close()
 
 
 async def test_data_stream_exc(loop, conn) -> None:
     fut = loop.create_future()
 
-    @async_generator
     async def gen():
-        await yield_(b"binary data")
+        yield b"binary data"
         await fut
 
     req = ClientRequest("POST", URL("http://python.org/"), data=gen(), loop=loop)
@@ -1017,9 +1030,10 @@ async def test_data_stream_exc(loop, conn) -> None:
 async def test_data_stream_exc_chain(loop, conn) -> None:
     fut = loop.create_future()
 
-    @async_generator
     async def gen():
         await fut
+        return
+        yield
 
     req = ClientRequest("POST", URL("http://python.org/"), data=gen(), loop=loop)
 
@@ -1043,10 +1057,9 @@ async def test_data_stream_exc_chain(loop, conn) -> None:
 
 
 async def test_data_stream_continue(loop, buf, conn) -> None:
-    @async_generator
     async def gen():
-        await yield_(b"binary data")
-        await yield_(b" result")
+        yield b"binary data"
+        yield b" result"
 
     req = ClientRequest(
         "POST", URL("http://python.org/"), data=gen(), expect100=True, loop=loop
@@ -1088,10 +1101,9 @@ async def test_data_continue(loop, buf, conn) -> None:
 
 
 async def test_close(loop, buf, conn) -> None:
-    @async_generator
     async def gen():
         await asyncio.sleep(0.00001)
-        await yield_(b"result")
+        yield b"result"
 
     req = ClientRequest("POST", URL("http://python.org/"), data=gen(), loop=loop)
     resp = await req.send(conn)
@@ -1127,6 +1139,8 @@ async def test_oserror_on_write_bytes(loop, conn) -> None:
     exc = conn.protocol.set_exception.call_args[0][0]
     assert isinstance(exc, aiohttp.ClientOSError)
 
+    await req.close()
+
 
 async def test_terminate(loop, conn) -> None:
     req = ClientRequest("get", URL("http://python.org"), loop=loop)
@@ -1139,6 +1153,8 @@ async def test_terminate(loop, conn) -> None:
     writer.cancel.assert_called_with()
     resp.close()
 
+    await req.close()
+
 
 def test_terminate_with_closed_loop(loop, conn) -> None:
     req = resp = writer = None
@@ -1154,6 +1170,7 @@ def test_terminate_with_closed_loop(loop, conn) -> None:
 
     loop.run_until_complete(go())
 
+    loop.run_until_complete(req.close())
     loop.close()
     req.terminate()
     assert req._writer is None
@@ -1168,6 +1185,8 @@ def test_terminate_without_writer(loop) -> None:
     req.terminate()
     assert req._writer is None
 
+    loop.run_until_complete(req.close())
+
 
 async def test_custom_req_rep(loop) -> None:
     conn = None
@@ -1266,3 +1285,5 @@ def test_loose_cookies_types(loop) -> None:
 
     for loose_cookies_type in accepted_types:
         req.update_cookies(cookies=loose_cookies_type)
+
+    loop.run_until_complete(req.close())
