@@ -12,6 +12,7 @@ import pytest
 from re_assert import Matches
 
 from aiohttp import streams
+from aiohttp.helpers import PY_310
 
 DATA = b"line1\nline2\nline3\n"
 
@@ -24,7 +25,7 @@ def chunkify(seq, n):
 async def create_stream():
     loop = asyncio.get_event_loop()
     protocol = mock.Mock(_reading_paused=False)
-    stream = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    stream = streams.StreamReader(protocol, 2**16, loop=loop)
     stream.feed_data(DATA)
     stream.feed_eof()
     return stream
@@ -72,7 +73,7 @@ class TestStreamReader:
     DATA = b"line1\nline2\nline3\n"
 
     def _make_one(self, *args, **kwargs):
-        kwargs.setdefault("limit", 2 ** 16)
+        kwargs.setdefault("limit", 2**16)
         return streams.StreamReader(mock.Mock(_reading_paused=False), *args, **kwargs)
 
     async def test_create_waiter(self) -> None:
@@ -82,12 +83,25 @@ class TestStreamReader:
         with pytest.raises(RuntimeError):
             await stream._wait("test")
 
+    @pytest.mark.xfail(
+        PY_310,
+        reason="No idea why ClientRequest() is constructed out of loop but "
+        "it calls `asyncio.get_event_loop()`",
+        raises=DeprecationWarning,
+    )
     def test_ctor_global_loop(self) -> None:
         loop = asyncio.new_event_loop()
         asyncio.set_event_loop(loop)
-        stream = streams.StreamReader(mock.Mock(_reading_paused=False), 2 ** 16)
+        try:
+            stream = streams.StreamReader(mock.Mock(_reading_paused=False), 2**16)
 
-        assert stream._loop is loop
+            assert stream._loop is loop
+        finally:  # Otherwise an unstopped/unclosed loop affects the next test
+            # Cleanup, leaks into `test_at_eof` otherwise:
+            loop.stop()
+            loop.run_forever()
+            loop.close()
+            gc.collect()
 
     async def test_at_eof(self) -> None:
         stream = self._make_one()
@@ -394,6 +408,117 @@ class TestStreamReader:
         with pytest.raises(ValueError):
             await stream.readline()
 
+    async def test_readuntil(self) -> None:
+        loop = asyncio.get_event_loop()
+        # Read one chunk. 'readuntil' will need to wait for the data
+        # to come from 'cb'
+        stream = self._make_one()
+        stream.feed_data(b"chunk1 ")
+        read_task = loop.create_task(stream.readuntil(b"*"))
+
+        def cb():
+            stream.feed_data(b"chunk2 ")
+            stream.feed_data(b"chunk3 ")
+            stream.feed_data(b"* chunk4")
+
+        loop.call_soon(cb)
+
+        line = await read_task
+        assert b"chunk1 chunk2 chunk3 *" == line
+
+        stream.feed_eof()
+        data = await stream.read()
+        assert b" chunk4" == data
+
+    async def test_readuntil_limit_with_existing_data(self) -> None:
+        # Read one chunk. The data is in StreamReader's buffer
+        # before the event loop is run.
+
+        stream = self._make_one(limit=2)
+        stream.feed_data(b"li")
+        stream.feed_data(b"ne1&line2&")
+
+        with pytest.raises(ValueError):
+            await stream.readuntil(b"&")
+        # The buffer should contain the remaining data after exception
+        stream.feed_eof()
+        data = await stream.read()
+        assert b"line2&" == data
+
+    async def test_readuntil_limit(self) -> None:
+        loop = asyncio.get_event_loop()
+        # Read one chunk. StreamReaders are fed with data after
+        # their 'readuntil' methods are called.
+        stream = self._make_one(limit=4)
+
+        def cb():
+            stream.feed_data(b"chunk1")
+            stream.feed_data(b"chunk2$")
+            stream.feed_data(b"chunk3#")
+            stream.feed_eof()
+
+        loop.call_soon(cb)
+
+        with pytest.raises(ValueError):
+            await stream.readuntil(b"$")
+        data = await stream.read()
+        assert b"chunk3#" == data
+
+    async def test_readuntil_nolimit_nowait(self) -> None:
+        # All needed data for the first 'readuntil' call will be
+        # in the buffer.
+        stream = self._make_one()
+        data = b"line1!line2!line3!"
+        stream.feed_data(data[:6])
+        stream.feed_data(data[6:])
+
+        line = await stream.readuntil(b"!")
+        assert b"line1!" == line
+
+        stream.feed_eof()
+        data = await stream.read()
+        assert b"line2!line3!" == data
+
+    async def test_readuntil_eof(self) -> None:
+        stream = self._make_one()
+        stream.feed_data(b"some data")
+        stream.feed_eof()
+
+        line = await stream.readuntil(b"@")
+        assert b"some data" == line
+
+    async def test_readuntil_empty_eof(self) -> None:
+        stream = self._make_one()
+        stream.feed_eof()
+
+        line = await stream.readuntil(b"@")
+        assert b"" == line
+
+    async def test_readuntil_read_byte_count(self) -> None:
+        stream = self._make_one()
+        data = b"line1!line2!line3!"
+        stream.feed_data(data)
+
+        await stream.readuntil(b"!")
+
+        data = await stream.read(7)
+        assert b"line2!l" == data
+
+        stream.feed_eof()
+        data = await stream.read()
+        assert b"ine3!" == data
+
+    async def test_readuntil_exception(self) -> None:
+        stream = self._make_one()
+        stream.feed_data(b"line#")
+
+        data = await stream.readuntil(b"#")
+        assert b"line#" == data
+
+        stream.set_exception(ValueError())
+        with pytest.raises(ValueError):
+            await stream.readuntil(b"#")
+
     async def test_readexactly_zero_or_less(self) -> None:
         # Read exact number of bytes (zero or less).
         stream = self._make_one()
@@ -1123,7 +1248,7 @@ class TestDataQueue:
 
 async def test_feed_data_waiters(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1140,7 +1265,7 @@ async def test_feed_data_waiters(protocol) -> None:
 
 async def test_feed_data_completed_waiters(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     waiter = reader._waiter = loop.create_future()
 
     waiter.set_result(1)
@@ -1151,7 +1276,7 @@ async def test_feed_data_completed_waiters(protocol) -> None:
 
 async def test_feed_eof_waiters(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1166,7 +1291,7 @@ async def test_feed_eof_waiters(protocol) -> None:
 
 async def test_feed_eof_cancelled(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1183,7 +1308,7 @@ async def test_feed_eof_cancelled(protocol) -> None:
 
 async def test_on_eof(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
 
     on_eof = mock.Mock()
     reader.on_eof(on_eof)
@@ -1204,7 +1329,7 @@ async def test_on_eof_empty_reader() -> None:
 
 async def test_on_eof_exc_in_callback(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
 
     on_eof = mock.Mock()
     on_eof.side_effect = ValueError
@@ -1228,7 +1353,7 @@ async def test_on_eof_exc_in_callback_empty_stream_reader() -> None:
 
 async def test_on_eof_eof_is_set(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     reader.feed_eof()
 
     on_eof = mock.Mock()
@@ -1239,7 +1364,7 @@ async def test_on_eof_eof_is_set(protocol) -> None:
 
 async def test_on_eof_eof_is_set_exception(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     reader.feed_eof()
 
     on_eof = mock.Mock()
@@ -1252,7 +1377,7 @@ async def test_on_eof_eof_is_set_exception(protocol) -> None:
 
 async def test_set_exception(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1267,7 +1392,7 @@ async def test_set_exception(protocol) -> None:
 
 async def test_set_exception_cancelled(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
     waiter = reader._waiter = loop.create_future()
     eof_waiter = reader._eof_waiter = loop.create_future()
 
@@ -1285,7 +1410,7 @@ async def test_set_exception_cancelled(protocol) -> None:
 
 async def test_set_exception_eof_callbacks(protocol) -> None:
     loop = asyncio.get_event_loop()
-    reader = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    reader = streams.StreamReader(protocol, 2**16, loop=loop)
 
     on_eof = mock.Mock()
     reader.on_eof(on_eof)
@@ -1325,7 +1450,7 @@ async def test_data_queue_empty() -> None:
     buffer = streams.DataQueue(loop)
     buffer.feed_eof()
 
-    async for _ in buffer:  # NOQA
+    async for _ in buffer:
         assert False
 
 
@@ -1368,7 +1493,7 @@ async def test_stream_reader_iter_chunks_no_chunked_encoding() -> None:
 
 async def test_stream_reader_iter_chunks_chunked_encoding(protocol) -> None:
     loop = asyncio.get_event_loop()
-    stream = streams.StreamReader(protocol, 2 ** 16, loop=loop)
+    stream = streams.StreamReader(protocol, 2**16, loop=loop)
     for line in DATA.splitlines(keepends=True):
         stream.begin_http_chunk_receiving()
         stream.feed_data(line)
@@ -1379,3 +1504,7 @@ async def test_stream_reader_iter_chunks_chunked_encoding(protocol) -> None:
     async for data, end_of_chunk in stream.iter_chunks():
         assert (data, end_of_chunk) == (next(it), True)
     pytest.raises(StopIteration, next, it)
+
+
+def test_isinstance_check() -> None:
+    assert isinstance(streams.EMPTY_PAYLOAD, streams.StreamReader)
